@@ -9,7 +9,7 @@
 #
 package MooseX::AttributeShortcuts;
 {
-  $MooseX::AttributeShortcuts::VERSION = '0.015';
+  $MooseX::AttributeShortcuts::VERSION = '0.017';
 }
 
 # ABSTRACT: Shorthand for common attribute options
@@ -22,6 +22,7 @@ use namespace::autoclean;
 use Moose ();
 use Moose::Exporter;
 use Moose::Util::MetaRole;
+use Moose::Util::TypeConstraints;
 
 # debug...
 #use Smart::Comments;
@@ -29,14 +30,16 @@ use Moose::Util::MetaRole;
 {
     package MooseX::AttributeShortcuts::Trait::Attribute;
 {
-  $MooseX::AttributeShortcuts::Trait::Attribute::VERSION = '0.015';
+  $MooseX::AttributeShortcuts::Trait::Attribute::VERSION = '0.017';
 }
     use namespace::autoclean;
     use MooseX::Role::Parameterized;
-
+    use Moose::Util::TypeConstraints qw{ class_type role_type enum };
     use MooseX::Types::Moose          ':all';
     use MooseX::Types::Common::String ':all';
 
+    sub _find_or_create_isa_type_constraint { goto \&Moose::Util::TypeConstraints::find_or_create_isa_type_constraint }
+
     parameter writer_prefix  => (isa => NonEmptySimpleStr, default => '_set_');
     parameter builder_prefix => (isa => NonEmptySimpleStr, default => '_build_');
 
@@ -67,11 +70,27 @@ use Moose::Util::MetaRole;
             init_arg  => '_anon_builder',
         );
 
+        has constraint => (
+            is        => 'ro',
+            isa       => 'CodeRef',
+            predicate => 'has_constraint',
+        );
+
+        has original_isa => (
+            is        => 'ro',
+            predicate => 'has_original_isa',
+        );
+
+        # TODO coerce via, transform ?
+
+        # has original_isa, original_coerce ?
+
         my $_process_options = sub {
             my ($class, $name, $options) = @_;
 
-            my $_has = sub { defined $options->{$_[0]} };
+            my $_has = sub { defined $options->{$_[0]}             };
             my $_opt = sub { $_has->(@_) ? $options->{$_[0]} : q{} };
+            my $_ref = sub { ref $_opt->(@_) || q{}                };
 
             if ($options->{is}) {
 
@@ -90,26 +109,65 @@ use Moose::Util::MetaRole;
                 }
             }
 
+            # TODO isa_class - anon class_type generation
+            # TODO isa_role  - anon role_type generation
+            # TODO isa_enum  - anon enum generation
+            # TODO coerce_via - anon coercion (type -> anon subtype+coercion
+
+            # XXX we also ignore conflicts here -- last in wins
+            #confess q{conflict 'isa' and 'isa_class' or 'isa_role'}
+                #if $_has->('isa')
+
+            # XXX undocumented -- not sure this is a great idea
+            $options->{isa} = class_type(delete $options->{isa_class})
+                if $_has->('isa_class');
+            $options->{isa} = role_type(delete $options->{isa_role})
+                if $_has->('isa_role');
+            $options->{isa} = enum(delete $options->{isa_enum})
+                if $_has->('isa_enum');
+
+            $class->throw_error('You must specify an isa when declaring a constraint')
+                if $_has->('constraint') && !$_has->('isa');
+
+            if (my $isa = $_opt->('isa')) {
+
+                my @opts;
+
+                # constraint checking! XXX message, etc?
+                push @opts, constraint => $_opt->('constraint')
+                    if $_ref->('constraint') eq 'CODE';
+
+                if (@opts) {
+
+                    # stash our original option away and construct our new one
+                    $options->{original_isa} = $isa;
+                    $options->{isa}
+                        = _find_or_create_isa_type_constraint($isa)
+                        ->create_child_type(@opts)
+                        ;
+                }
+            }
+
             if ($options->{lazy_build} && $options->{lazy_build} eq 'private') {
 
                 $options->{lazy_build} = 1;
                 $options->{clearer}    = "_clear_$name";
                 $options->{predicate}  = "_has_$name";
-                $options->{init_arg}   = "_$name"
-                    unless exists $options->{init_arg};
             }
 
             my $is_private = sub { $name =~ /^_/ ? $_[0] : $_[1] };
             my $default_for = sub {
                 my ($opt) = @_;
 
-                if ($options->{$opt} && $options->{$opt} eq '1') {
-                    $options->{$opt} =
-                        $is_private->('_', q{}) .
-                        $prefix{$opt} .
-                        $is_private->(q{}, '_') .
-                        $name;
-                }
+                return unless $_has->($opt);
+                my $opt_val = $_opt->($opt);
+
+                my ($head, $mid)
+                    = $opt_val eq '1'  ? ($is_private->('_', q{}), $is_private->(q{}, '_'))
+                    : $opt_val eq '-1' ? ($is_private->(q{}, '_'), $is_private->(q{}, '_'))
+                    :                    return;
+
+                $options->{$opt} = $head . $prefix{$opt} . $mid . $name;
                 return;
             };
 
@@ -223,6 +281,7 @@ sub init_meta {
     Moose::Util::MetaRole::apply_metaroles(
         for             => $for_class,
         class_metaroles => { attribute         => [ $role ] },
+        # TODO add attribute trait here to create builder method if found
         role_metaroles  => { applied_attribute => [ $role ] },
     );
 
@@ -231,7 +290,7 @@ sub init_meta {
 
 1;
 
-
+__END__
 
 =pod
 
@@ -245,7 +304,7 @@ MooseX::AttributeShortcuts - Shorthand for common attribute options
 
 =head1 VERSION
 
-This document describes version 0.015 of MooseX::AttributeShortcuts - released August 26, 2012 as part of MooseX-AttributeShortcuts.
+This document describes version 0.017 of MooseX::AttributeShortcuts - released October 28, 2012 as part of MooseX-AttributeShortcuts.
 
 =head1 SYNOPSIS
 
@@ -337,7 +396,8 @@ Unless specified here, all options defined by L<Moose::Meta::Attribute> and
 L<Class::MOP::Attribute> remain unchanged.
 
 Want to see additional options?  Ask, or better yet, fork on GitHub and send
-a pull request.
+a pull request. If the shortcuts you're asking for already exist in L<Moo> or
+L<Mouse> or elsewhere, please note that as it will carry significant weight.
 
 For the following, "$name" should be read as the attribute name; and the
 various prefixes should be read using the defaults.
@@ -437,6 +497,23 @@ e.g., in your class,
     has foo => (is => 'ro', builder => '_build_foo');
     sub _build_foo { 'bar!' }
 
+=head2 constraint => sub { ... }
+
+Specifying the constraint option with a coderef will cause a new type
+constraint to be created, with the parent type being the type specified in the
+C<isa> option and the constraint being the coderef supplied here.
+
+Example:
+
+    # value must be an integer greater than 10 to pass the constraint
+    has thinger => (
+        isa        => 'Int',
+        constraint => sub { $_ > 10 },
+        # ...
+    );
+
+Note that if you supply a constraint, you must also provide an C<isa>.
+
 =for Pod::Coverage init_meta
 
 =head1 SOURCE
@@ -466,7 +543,3 @@ This is free software, licensed under:
   The GNU Lesser General Public License, Version 2.1, February 1999
 
 =cut
-
-
-__END__
-
